@@ -34,6 +34,12 @@ import pkg_resources
 
 import gradio as gr
 
+from CustomIntents.Pfunction.Pfunctions import string_value_check, float_value_check, boolean_value_check, \
+    int_value_check, count_subdictionaries
+
+available_models = ["s1", "s1a", "s2", "s3", "m1", "m2", "l1", "vgg-19", "l1.1", "l2", "x1"]
+available_optimizers = ["adam"]
+
 
 class VideoStream:
 
@@ -61,9 +67,30 @@ class VideoStream:
 
 
 class ImageClassificator:
-    def __init__(self, data_folder="data", model_name="imageclassification_model", number_of_classes=2,
+    def __init__(self, data_folder="data",
+                 model_name="imageclassification_model",
+                 number_of_classes=2,
                  classes_name=None,
-                 gpu=None):
+                 gpu=None,
+                 checkpoint_filepath='/tmp/checkpoint_epoch:{epoch}'):
+
+        ###############################
+        # check that inputs are valid #
+        ###############################
+        if not string_value_check(data_folder):
+            raise ValueError("data_folder must be a string")
+        if not string_value_check(model_name):
+            raise ValueError("model_name must be a string")
+        if not int_value_check(number_of_classes, start=2):
+            raise ValueError("number_of_classes must be a int and greater or equal to 2")
+        if type(classes_name) is not list or classes_name is not None:
+            raise ValueError("classes_name must be a list")
+        if not boolean_value_check(gpu):
+            raise ValueError("gpu must be a boolean")
+        if not string_value_check(checkpoint_filepath):
+            raise ValueError("checkpoint_filepath must be a string")
+        ################################
+
         self.optimizer = None
         self.acc = None
         self.re = None
@@ -82,14 +109,26 @@ class ImageClassificator:
         self.hist = None
         self.clases = None
         self.data = None
+        self.callbacks = []
+        self.checkpoint_filepath = checkpoint_filepath
         self.number_of_classes = number_of_classes
         self._name_the_classes(classes_name)
         self.gpu_usage = gpu
         self._check_for_gpu_availability()
+        self._check_the_data_folder()
         self.data_folder = data_folder
         self.name = model_name
         self._oom_avoider()
 
+    def _check_the_data_folder(self):
+        data_folde_exists = os.path.exists(self.data_folder)
+        if not data_folde_exists:
+            raise ValueError("data_folder does not exist")
+        number_of_subdirectories = count_subdictionaries(self.data_folder)
+        if not number_of_subdirectories == self.number_of_classes:
+            raise ValueError(
+                "number of subdirectories in data_folder is not equal to number of classes please check you data folder")
+
     def _name_the_classes(self, classes_name):
         self.clases = [x for x in range(self.number_of_classes)]
         if classes_name is not None:
@@ -133,8 +172,8 @@ class ImageClassificator:
         self.data = self.data.map(lambda x, y: (x / 255, y))
         print(f"{bcolors.OKGREEN}scaling data succsesfuly{bcolors.ENDC}")
 
-    def _augmanet_data(self, model_type="s1"):
-        if "a" in model_type:
+    def _augmanet_data(self, model_type="s1", augment_data=False):
+        if "a" in model_type or augment_data:
             data_augmentation = Sequential([
                 tf.keras.layers.RandomFlip(mode="horizontal"),
                 tf.keras.layers.RandomRotation((-0.3, 0.3)),
@@ -145,7 +184,7 @@ class ImageClassificator:
                                       num_parallel_calls=tf.data.AUTOTUNE)
             print(f"{bcolors.OKGREEN}augmenting data succsesfuly{bcolors.ENDC}")
 
-    def _split_data(self, validation_split=0.2, test_split=0):
+    def _split_data(self, validation_split=0.2, test_split=0.0):
         self.train_size = int(len(self.data) * (1 - validation_split - test_split))
         self.val_size = int(len(self.data) * validation_split)
         self.test_size = int(len(self.data) * test_split)
@@ -256,6 +295,25 @@ class ImageClassificator:
             self.model.add(Dense(self.number_of_classes, activation='softmax'))
             succsesful = True
 
+        elif model_type == "m2":
+            self.model = Sequential()
+            self.model.add(Conv2D(16, (5, 5), 1, padding="same", activation='relu', input_shape=(256, 256, 3)))
+            self.model.add(Conv2D(32, (3, 3), 1, padding="same", activation='relu'))
+            self.model.add(Conv2D(64, (3, 3), 1, activation='relu'))
+            self.model.add(MaxPooling2D())
+            self.model.add(Dropout(0.25))
+            self.model.add(Conv2D(128, (1, 1), 1, padding="same", activation='relu'))
+            self.model.add(Conv2D(128, (3, 3), 1, padding="same", activation='relu'))
+            self.model.add(Conv2D(64, (3, 3), 1, padding="same", activation='relu'))
+            self.model.add(MaxPooling2D())
+            self.model.add(Conv2D(32, (3, 3), 1, padding="same", activation='relu'))
+            self.model.add(Dropout(0.25))
+            self.model.add(Flatten())
+            self.model.add(Dense(512, activation='relu'))
+            self.model.add(Dropout(0.5))
+            self.model.add(Dense(self.number_of_classes, activation='softmax'))
+            succsesful = True
+
         elif model_type == "l1" or model_type.lower() == "vgg-19":
             self.model = Sequential()
             self.model.add(Conv2D(64, (3, 3), 1, padding="same", activation='relu', input_shape=(256, 256, 3)))
@@ -342,12 +400,6 @@ class ImageClassificator:
             self.model = self._make_small_Xception_model(input_shape=(256, 256, 3))
             succsesful = True
 
-        else:
-            print(f"{bcolors.FAIL}model {model_type} is undifinde\n"
-                  f"it will defuat to s1 {bcolors.ENDC}")
-            self._build_model(model_type="s1", optimizer=optimizer)
-            succsesful = False
-
         if succsesful:
             print(self.model.summary())
 
@@ -374,29 +426,88 @@ class ImageClassificator:
                 path = os.path.join(parent_dir, self.logdir)
                 os.mkdir(path)
 
-    def train_model(self, epochs=20, model_type="s1", logdir=None, optimizer_type="adam", learning_rate=0.00001,
-                    class_weight=None, prefetching=False, plot_model=True, validation_split=0.2, test_split=0):
+    def _adding_checkpoint(self):
+        model_checkpoint_callback = tf.keras.callbacks.ModelCheckpoint(
+            filepath=self.checkpoint_filepath,
+            save_weights_only=True,
+            monitor='val_accuracy',
+            mode='max',
+            verbose=1)
+        self.callbacks.append(model_checkpoint_callback)
+
+    def train_model(self, epochs=20,
+                    model_type="s1",
+                    logdir=None,
+                    optimizer_type="adam",
+                    learning_rate=0.00001,
+                    class_weight=None,
+                    prefetching=False,
+                    plot_model=True,
+                    validation_split=0.2,
+                    test_split=float(0),
+                    augment_data=False,
+                    tensorboard_usage=False,
+                    stop_early=False,
+                    checkpoint=False):
+
+        ###############################
+        # check that inputs are valid #
+        ###############################
         if type(epochs) is not int:
             print(f"{bcolors.FAIL}epochs should be an int\n"
                   f"it will defualt to 20{bcolors.ENDC}")
             epochs = 20
+        if not string_value_check(model_type, valids=available_models):
+            raise ValueError(f"model type must be a string and one of the following values {available_models}")
+        if not string_value_check(logdir) and logdir is not None:
+            raise ValueError("logdir must be a directory")
+        if not string_value_check(optimizer_type, valids=available_optimizers):
+            raise ValueError(f"optimizer type must be a string and one of the following valeus {available_optimizers}")
+        if not float_value_check(learning_rate):
+            raise ValueError("learning rate must be a float")
+        if type(class_weight) is not dict and class_weight is not None:
+            raise ValueError("class weight must be a dict")
+        if type(prefetching) is not bool:
+            raise ValueError("prefetching must be a boolean")
+        if type(plot_model) is not bool:
+            raise ValueError("plot_model must be a boolean")
+        if not float_value_check(validation_split, start=0, end=0.99):
+            raise ValueError("validation split must be a float and between 0 and 0.99")
+        if not float_value_check(test_split, start=0, end=0.99):
+            raise ValueError("test split must be a float and between 0 and 0.99")
+        if type(augment_data) is not bool:
+            raise ValueError("augment_data must be a boolean")
+        if type(tensorboard_usage) is not bool:
+            raise ValueError("tensorboard_usage must be a boolean")
+        if type(stop_early) is not bool:
+            raise ValueError("stop_early must be a boolean")
+        if type(checkpoint) is not bool:
+            raise ValueError("checkpoint must be a boolean")
+        ###############################
+
         self._oom_avoider()
         self._remove_dogy_images()
         self._load_data()
         self._scale_data()
-        self._augmanet_data(model_type=model_type)
+        self._augmanet_data(model_type=model_type, augment_data=augment_data)
         self._split_data(validation_split=validation_split, test_split=test_split)
         if prefetching:
             self._prefetching_data()
-        self.logdir = logdir
-        self._seting_logdir()
+        if tensorboard_usage:
+            self.logdir = logdir
+            self._seting_logdir()
         self._build_optimizer(optimizer_type=optimizer_type, learning_rate=learning_rate)
         self._build_model(model_type=model_type, optimizer=self.optimizer)
         if plot_model:
             tf.keras.utils.plot_model(self.model, show_shapes=True, show_layer_activations=True)
-        self.tensorboard_callback = tf.keras.callbacks.TensorBoard(log_dir=self.logdir)
+        if tensorboard_usage:
+            self.callbacks.append(tf.keras.callbacks.TensorBoard(log_dir=self.logdir))
+        if stop_early:
+            self.callbacks.append(tf.callbacks.EarlyStopping(monitor="val_loss", patience=5, restore_best_weights=True))
+        if checkpoint:
+            self._adding_checkpoint()
         self.hist = self.model.fit(self.train, epochs=epochs, validation_data=self.val,
-                                   callbacks=[self.tensorboard_callback], class_weight=class_weight)
+                                   callbacks=self.callbacks, class_weight=class_weight)
         self._plot_acc()
         self._plot_loss()
 
@@ -419,6 +530,8 @@ class ImageClassificator:
         plt.show()
 
     def save_model(self, model_file_name=None):
+        if not string_value_check(model_file_name):
+            model_file_name = str(model_file_name)
         if model_file_name is None:
             model_file_name = self.name
         if model_file_name.endswith(".h5"):
@@ -427,6 +540,8 @@ class ImageClassificator:
             self.model.save(f"{model_file_name}.h5")
 
     def load_model(self, name="imageclassification_model"):
+        if not string_value_check(name):
+            name = str(name)
         if name.endswith(".h5"):
             self.model = load_model(name)
         else:
@@ -438,7 +553,8 @@ class ImageClassificator:
             for gpu in gpus:
                 tf.config.experimental.set_memory_growth(gpu, True)
 
-    def predict(self, image, image_type=None, full_mode=False, accuracy=False):
+    def predict(self, image, image_type=None,
+                full_mode=False, accuracy=False):
         if image_type is None and type(image) is str:
             image_type = "path"
         if image_type == "path":
@@ -461,21 +577,119 @@ class ImageClassificator:
             else:
                 return result_class
 
-    def _fast_predict(self, img):
-        res = self.model.predict(img, verbose=0)
-        class_index = res.argmax()
-        result_class = self.clases[class_index]
-        result_percentage = res[0][class_index]
-        return result_class, result_percentage
+    def predict_face(self, img,
+                     image_type=None,
+                     full_mode=False,
+                     accuracy=False,
+                     return_picture=False,
+                     save_returned_picture=False,
+                     saved_returned_picture_name="test.jpg",
+                     show_returned_picture=False):
+        if not return_picture:
+            haar_cascade_file = pkg_resources.resource_filename(__name__, "cascades/haarcascade_frontalcatface.xml")
+            detector = cv2.CascadeClassifier(haar_cascade_file)
+            if image_type is None and type(image) is str:
+                image_type = "path"
+            if image_type == "path":
+                img = cv2.imread(image)
+            if image_type == "numpy256":
+                img = img / 256
+            gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
+            rects = detector.detectMultiScale(gray, scaleFactor=1.1,
+                                              minNeighbors=5, minSize=(50, 50),
+                                              flags=cv2.CASCADE_SCALE_IMAGE)
+            # loop over the face bounding boxes
+            for (fX, fY, fW, fH) in rects:
+                # extract the ROI of the face from the grayscale image,
+                # resize it to a fixed 28x28 pixels, and then prepare the
+                # ROI for classification via the CNN
+                roi = img[fY:fY + fH, fX:fX + fW]
+                roi = cv2.resize(roi, (256, 256))
+                roi = roi.astype("float") / 255.0
+                roi = img_to_array(roi)
+                roi = np.expand_dims(roi, axis=0)
+                res = self.model.predict(roi)
+                if full_mode:
+                    res = np.array([res[0],
+                                    self.clases])
+                    res = res.transpose()
+                    return res
+                else:
+                    class_index = res.argmax()
+                    result_class = self.clases[class_index]
+                    result_percentage = res[0][class_index]
+                    if accuracy:
+                        return result_class, result_percentage
+                    else:
+                        return result_class
+        else:
+            haar_cascade_file = pkg_resources.resource_filename(__name__, "cascades/haarcascade_frontalcatface.xml")
+            detector = cv2.CascadeClassifier(haar_cascade_file)
+            frame = img
+            label = ""
+            # keep looping
+            frame_count = 0
 
-    def gradio_preview(self, share=False, inbrowser=True):
-        demo = gr.Interface(self.predict,
-                            inputs=gr.Image(label="your image"),
-                            outputs=[gr.Label(label="class"),
-                                     gr.Label(label="Accuracy")],
-                            allow_flagging="never")
-        print(f"open http://localhost:7860 for viewing your model preview")
-        demo.launch(share=share, inbrowser=inbrowser)
+            # grab the current frame
+            gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
+            frameClone = frame.copy()
+            rects = detector.detectMultiScale(gray, scaleFactor=1.3,
+                                              minNeighbors=5, minSize=(50, 50),
+                                              flags=cv2.CASCADE_SCALE_IMAGE)
+            # loop over the face bounding boxes
+            for (fX, fY, fW, fH) in rects:
+                # extract the ROI of the face from the grayscale image,
+                # resize it to a fixed 28x28 pixels, and then prepare the
+                # ROI for classification via the CNN
+                roi = frame[fY:fY + fH, fX:fX + fW]
+                roi = cv2.resize(roi, (256, 256))
+                roi = roi.astype("float") / 255.0
+                roi = img_to_array(roi)
+                roi = np.expand_dims(roi, axis=0)
+                prediction = str(self._fast_predict(roi))
+                label = prediction
+                cv2.putText(frameClone, label, (fX, fY - 10),
+                            cv2.FONT_HERSHEY_SIMPLEX, 0.45, (0, 0, 255), 2)
+                cv2.rectangle(frameClone, (fX, fY), (fX + fW, fY + fH),
+                              (0, 0, 255), 2)
+                # show our detected faces along with smiling/not smiling labels
+            if save_returned_picture:
+                if saved_returned_picture_name.endswith(".jpg") or saved_returned_picture_name.endswith(".png"):
+                    cv2.imwrite(saved_returned_picture_name, frameClone)
+                else:
+                    cv2.imwrite(f"{saved_returned_picture_name}.jpg", frameClone)
+            if show_returned_picture:
+                cv2.imshow("Face", frameClone)
+            return frameClone
+
+    def _predict_face_for_gradio(self, img, accuracy):
+        output = self.predict_face(img=img, accuracy=accuracy, return_picture=True, save_returned_picture=False)
+        return output
+
+    def gradio_preview(self, share=False,
+                       inbrowser=True,
+                       predict_face=False):
+        # normoal preview
+        if not predict_face:
+            demo = gr.Interface(self.predict,
+                                inputs=gr.Image(label="your image"),
+                                outputs=[gr.Label(label="class"),
+                                         gr.Label(label="Accuracy")],
+                                allow_flagging="never")
+            print(f"open http://localhost:7860 for viewing your model preview")
+            demo.launch(share=share, inbrowser=inbrowser)
+        # face detection preview
+        else:
+            demo = gr.Interface(self._predict_face_for_gradio,
+                                inputs=[
+                                    gr.Image(label="your image"),
+                                    gr.Radio(label="printing accuracy")
+                                ],
+                                outputs=[
+                                    gr.Image(label="output")
+                                ], allow_flagging="never")
+            print(f"open http://localhost:7860 for viewing your model preview")
+            demo.launch(share=share, inbrowser=inbrowser)
 
     def evaluate_model(self):
         self.pre = Precision()
@@ -522,6 +736,13 @@ class ImageClassificator:
         cv2.destroyAllWindows()
         print("Done")
 
+    def _fast_predict(self, img):
+        res = self.model.predict(img, verbose=0)
+        class_index = res.argmax()
+        result_class = self.clases[class_index]
+        result_percentage = res[0][class_index]
+        return result_class, result_percentage
+
     def realtime_face_prediction(self, src=0, frame_rate=10):
         haar_cascade_file = pkg_resources.resource_filename(__name__, "cascades/haarcascade_frontalcatface.xml")
         detector = cv2.CascadeClassifier(haar_cascade_file)
